@@ -1,13 +1,11 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from sql import Null
-from sql.conditionals import Case
-
-from trytond.model import ModelView, ModelSQL, fields
+from trytond.model import ModelView, ModelSQL, MatchMixin, fields, \
+    sequence_ordered
 from trytond.pyson import Eval, Get, If, Bool
 from trytond.pool import PoolMeta
 
-__all__ = ['Product', 'ProductBom']
+__all__ = ['Product', 'ProductBom', 'ProductionLeadTime']
 
 
 class Product:
@@ -22,6 +20,14 @@ class Product:
                         'type', 'service').in_(['service', None]))),
             },
         depends=['type'])
+    lead_times = fields.One2Many('production.lead_time',
+        'product', 'Lead Times', order=[('sequence', 'ASC'), ('id', 'ASC')],
+        states={
+            'invisible': (Eval('type', 'service').in_(['service', None])
+                & (Eval('_parent_template', {}).get(
+                        'type', 'service').in_(['service', None]))),
+            },
+        depends=['type'])
 
     @classmethod
     def __setup__(cls):
@@ -58,7 +64,7 @@ class Product:
         return super(Product, cls).copy(products, default=default)
 
 
-class ProductBom(ModelSQL, ModelView):
+class ProductBom(sequence_ordered(), ModelSQL, ModelView):
     'Product - BOM'
     __name__ = 'product.product-production.bom'
 
@@ -73,12 +79,6 @@ class ProductBom(ModelSQL, ModelView):
                     Eval('product', 0),
                     Get(Eval('_parent_product', {}), 'id', 0))),
             ], depends=['product'])
-    sequence = fields.Integer('Sequence')
-
-    @classmethod
-    def __setup__(cls):
-        super(ProductBom, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
 
     def get_rec_name(self, name):
         return self.bom.rec_name
@@ -87,7 +87,26 @@ class ProductBom(ModelSQL, ModelView):
     def search_rec_name(cls, name, clause):
         return [('bom.rec_name',) + tuple(clause[1:])]
 
-    @staticmethod
-    def order_sequence(tables):
-        table, _ = tables[None]
-        return [Case((table.sequence == Null, 0), else_=1), table.sequence]
+
+class ProductionLeadTime(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
+    'Production Lead Time'
+    __name__ = 'production.lead_time'
+
+    product = fields.Many2One('product.product', 'Product',
+        ondelete='CASCADE', select=True, required=True,
+        domain=[
+            ('type', '!=', 'service'),
+            ])
+    bom = fields.Many2One('production.bom', 'BOM', ondelete='CASCADE',
+        domain=[
+            ('output_products', '=', If(Bool(Eval('product')),
+                    Eval('product', -1),
+                    Get(Eval('_parent_product', {}), 'id', 0))),
+            ],
+        depends=['product'])
+    lead_time = fields.TimeDelta('Lead Time')
+
+    @classmethod
+    def __setup__(cls):
+        super(ProductionLeadTime, cls).__setup__()
+        cls._order.insert(0, ('product', 'ASC'))
